require 'mk_cal_jpl/compute'

module MkCalJpl
  class Calendar
    attr_reader :year, :month, :day, :jd, :jd_jst

    include MkCalJpl::Compute

    def initialize(bin_path, jst)
      @bin_path = bin_path
      @year, @month, @day = jst
      @jd = gc2jd(@year, @month, @day)
      @jd_jst = @jd + Const::JST_D
      @sekki24_tms = Const::SEKKI_24_TM.each_slice(7).to_a
      @saku_tms    = Const::SAKU_TM.each_slice(6).to_a
    end

    #=========================================================================
    # 年間休日一覧
    #=========================================================================
    def holidays
      return get_holidays(@year)
    end

    #=========================================================================
    # 休日
    #=========================================================================
    def holiday
      return compute_holiday(@year, @month, @day)
    end

    #=========================================================================
    # 二十四節気
    #=========================================================================
    def sekki_24
      return compute_sekki_24(@jd_jst)
    end

    #=========================================================================
    # 雑節
    #=========================================================================
    def zassetsu
      return compute_zassetsu(@jd_jst)
    end

    #=========================================================================
    # 曜日
    #=========================================================================
    def yobi
      return compute_yobi(@jd_jst)
    end

    #=========================================================================
    # 干支
    #=========================================================================
    def kanshi
      return compute_kanshi(@jd_jst)
    end

    #=========================================================================
    # 節句
    #=========================================================================
    def sekku
      return compute_sekku(@month, @day)
    end

    #=========================================================================
    # 視黄経（太陽）
    #=========================================================================
    def lambda
      return compute_lambda(@jd_jst)
    end

    #=========================================================================
    # 視黄経（月）
    #=========================================================================
    def alpha
      return compute_alpha(@jd_jst)
    end

    #=========================================================================
    # 月齢（正午）
    #=========================================================================
    def moonage
      return compute_moonage(@jd_jst)
    end

    #=========================================================================
    # 旧暦
    #=========================================================================
    def oc
      return compute_oc(@jd_jst)
    end
  end
end

