module MkCalJpl
  module Const
    USAGE     = %Q{[USAGE] `MkCalJpl.new("<JPL_BIN_PATH>"[, "YYYYMMDD"])`}
    MSG_ERR_1 = "[ERROR] Binary file could not be found!"
    MSG_ERR_2 = "[ERROR] Date format should be `YYYYMMDD`!"
    MSG_ERR_3 = "[ERROR] Date is invalid!"
    JST_D     = 0.375  # = 9.0 / 24.0
    TT_TAI    = 32.184
    YOBI   = ["日", "月", "火", "水", "木", "金", "土"]
    ROKUYO = ["大安", "赤口", "先勝", "友引", "先負", "仏滅"]
    KANSHI = [
      "甲子", "乙丑", "丙寅", "丁卯", "戊辰", "己巳", "庚午", "辛未", "壬申", "癸酉",
      "甲戌", "乙亥", "丙子", "丁丑", "戊寅", "己卯", "庚辰", "辛巳", "壬午", "癸未",
      "甲申", "乙酉", "丙戌", "丁亥", "戊子", "己丑", "庚寅", "辛卯", "壬辰", "癸巳",
      "甲午", "乙未", "丙申", "丁酉", "戊戌", "己亥", "庚子", "辛丑", "壬寅", "癸卯",
      "甲辰", "乙巳", "丙午", "丁未", "戊申", "己酉", "庚戌", "辛亥", "壬子", "癸丑",
      "甲寅", "乙卯", "丙辰", "丁巳", "戊午", "己未", "庚申", "辛酉", "壬戌", "癸亥"
    ]
    SEKKI_24 = [
      "春分", "清明", "穀雨", "立夏", "小満", "芒種",
      "夏至", "小暑", "大暑", "立秋", "処暑", "白露",
      "秋分", "寒露", "霜降", "立冬", "小雪", "大雪",
      "冬至", "小寒", "大寒", "立春", "雨水", "啓蟄"
    ]
    SEKKU = [
      [0, 1, 7, "人日"],
      [1, 3, 3, "上巳"],
      [2, 5, 5, "端午"],
      [3, 7, 7, "七夕"],
      [4, 9, 9, "重陽"]
    ]
    ZASSETSU = [
      "節分"      , "彼岸入(春)", "彼岸(春)"  , "彼岸明(春)",
      "社日(春)"  , "土用入(春)", "八十八夜"  , "入梅"      ,
      "半夏生"    , "土用入(夏)", "二百十日"  , "二百二十日",
      "彼岸入(秋)", "彼岸(秋)"  , "彼岸明(秋)", "社日(秋)"  ,
      "土用入(秋)", "土用入(冬)"
    ]
    HOLIDAY = [
      [ 0,  1,  1, 0, 1949, 9999, "元日"              ],
      [ 1,  1, 15, 0, 1949, 1999, "成人の日"          ],
      [ 2,  1,  0, 2, 2000, 9999, "成人の日"          ],
      [ 3,  2, 11, 0, 1967, 9999, "建国記念の日"      ],
      [ 4,  2, 23, 0, 2020, 9999, "天皇誕生日"        ],
      [ 5,  2, 24, 0, 1989, 1989, "昭和天皇の大喪の礼"],
      [ 6,  3,  0, 4, 1949, 9999, "春分の日"          ],
      [ 7,  4, 10, 0, 1959, 1959, "明仁親王の結婚の儀"],
      [ 8,  4, 29, 0, 1949, 1988, "天皇誕生日"        ],
      [ 9,  4, 29, 0, 1989, 2006, "みどりの日"        ],
      [10,  4, 29, 0, 2007, 9999, "昭和の日"          ],
      [11,  5,  1, 0, 2019, 2019, "天皇の即位の日"    ],
      [12,  5,  3, 0, 1949, 9999, "憲法記念日"        ],
      [13,  5,  4, 0, 2007, 9999, "みどりの日"        ],
      [14,  5,  5, 0, 1949, 9999, "こどもの日"        ],
      [15,  6,  9, 0, 1993, 1993, "徳仁親王の結婚の儀"],
      [16,  7, 20, 0, 1996, 2002, "海の日"            ],
      [17,  7, 23, 0, 2020, 2020, "海の日"            ],
      [18,  7, 24, 0, 2020, 2020, "スポーツの日"      ],
      [19,  7,  0, 3, 2003, 2019, "海の日"            ],
      [20,  7,  0, 3, 2021, 9999, "海の日"            ],
      [21,  8, 10, 0, 2020, 2020, "山の日"            ],
      [22,  8, 11, 0, 2016, 2019, "山の日"            ],
      [23,  8, 11, 0, 2021, 9999, "山の日"            ],
      [24,  9, 15, 0, 1966, 2002, "敬老の日"          ],
      [25,  9,  0, 3, 2003, 9999, "敬老の日"          ],
      [26,  9,  0, 4, 1948, 9999, "秋分の日"          ],
      [27, 10, 10, 0, 1966, 1999, "体育の日"          ],
      [28, 10, 22, 0, 2019, 2019, "即位礼正殿の儀"    ],
      [29, 10,  0, 2, 2000, 2019, "体育の日"          ],
      [30, 10,  0, 2, 2021, 9999, "スポーツの日"      ],
      [31, 11,  3, 0, 1948, 9999, "文化の日"          ],
      [32, 11, 12, 0, 1990, 1990, "即位礼正殿の儀"    ],
      [33, 11, 23, 0, 1948, 9999, "勤労感謝の日"      ],
      [34, 12, 23, 0, 1989, 2018, "天皇誕生日"        ],
      [90,  0,  0, 8, 1973, 9999, "振替休日"          ],  # 1973-04-12 以降
      [91,  0,  0, 9, 1986, 9999, "国民の休日"        ]
    ]
  end
end

